@@ -1,7 +1,7 @@
 # -*- coding: us-ascii -*-
+# frozen_string_literal: false
 require 'test/unit'
 require 'tempfile'
-require_relative '../ruby/envutil'
 
 if Process.euid == 0
   ok = true
@@ -10,11 +10,45 @@ elsif (sudo = ENV["SUDO"]) and !sudo.empty? and (`#{sudo} echo ok` rescue false)
 else
   ok = false
 end
+if ok
+  case RUBY_PLATFORM
+  when /darwin/i
+    begin
+      require 'pty'
+    rescue LoadError
+      ok = false
+    end
+  end
+end
 ok &= (`dtrace -V` rescue false)
 module DTrace
   class TestCase < Test::Unit::TestCase
     INCLUDE = File.expand_path('..', File.dirname(__FILE__))
 
+    case RUBY_PLATFORM
+    when /solaris/i
+      # increase bufsize to 8m (default 4m on Solaris)
+      DTRACE_CMD = %w[dtrace -b 8m]
+    when /darwin/i
+      READ_PROBES = proc do |cmd|
+        lines = nil
+        PTY.spawn(*cmd) do |io, _, pid|
+          lines = io.readlines.each {|line| line.sub!(/\r$/, "")}
+          Process.wait(pid)
+        end
+        lines
+      end
+    end
+
+    DTRACE_CMD ||= %w[dtrace]
+
+    READ_PROBES ||= proc do |cmd|
+      IO.popen(cmd, err: [:child, :out], &:readlines)
+    end
+
+    miniruby = "#{RbConfig::TOPDIR}/miniruby#{RbConfig::CONFIG["EXEEXT"]}"
+    RUBYBIN =  File.exist?(miniruby) ? miniruby : EnvUtil.rubybin
+
     def trap_probe d_program, ruby_program
       d = Tempfile.new(%w'probe .d')
       d.write d_program
@@ -27,7 +61,7 @@ module DTrace
       d_path  = d.path
       rb_path = rb.path
 
-      cmd = ["dtrace", "-q", "-s", d_path, "-c", "#{EnvUtil.rubybin} -I#{INCLUDE} #{rb_path}"]
+      cmd = [*DTRACE_CMD, "-q", "-s", d_path, "-c", "#{RUBYBIN} -I#{INCLUDE} #{rb_path}"]
       if sudo = @@sudo
         [RbConfig::CONFIG["LIBPATHENV"], "RUBY", "RUBYOPT"].each do |name|
           if name and val = ENV[name]
@@ -36,9 +70,7 @@ module DTrace
         end
         cmd.unshift(sudo)
       end
-      probes = IO.popen(cmd, err: [:child, :out]) do |io|
-        io.readlines
-      end
+      probes = READ_PROBES.(cmd)
       d.close(true)
       rb.close(true)
       yield(d_path, rb_path, probes)
