@@ -74,6 +74,12 @@ class TestRegexp < Test::Unit::TestCase
     end
   end
 
+  def test_to_s_extended_subexp
+    re = /#\g#{"\n"}/x
+    re = /#{re}/
+    assert_warn('', '[ruby-core:82328] [Bug #13798]') {re.to_s}
+  end
+
   def test_union
     assert_equal :ok, begin
       Regexp.union(
@@ -111,8 +117,10 @@ class TestRegexp < Test::Unit::TestCase
     assert_equal('#<MatchData "&amp; y" foo:"amp" foo:"y">',
       /&(?<foo>.*?); (?<foo>y)/.match("aaa &amp; yyy").inspect)
 
-    /(?<id>[A-Za-z_]+)/ =~ "!abc"
-    assert_equal("abc", Regexp.last_match(:id))
+    /(?<_id>[A-Za-z_]+)/ =~ "!abc"
+    assert_not_nil(Regexp.last_match)
+    assert_equal("abc", Regexp.last_match(1))
+    assert_equal("abc", Regexp.last_match(:_id))
 
     /a/ =~ "b" # doesn't match.
     assert_equal(nil, Regexp.last_match)
@@ -142,6 +150,8 @@ class TestRegexp < Test::Unit::TestCase
     assert_equal("a[b]c", "abc".sub(/(?<x>[bc])/, "[\\k<x>]"))
 
     assert_equal("o", "foo"[/(?<bar>o)/, "bar"])
+    assert_equal("o", "foo"[/(?<@bar>o)/, "@bar"])
+    assert_equal("o", "foo"[/(?<@bar>.)\g<@bar>\k<@bar>/, "@bar"])
 
     s = "foo"
     s[/(?<bar>o)/, "bar"] = "baz"
@@ -173,8 +183,23 @@ class TestRegexp < Test::Unit::TestCase
     assert_raise(IndexError, bug9903) {m[key.dup.force_encoding(Encoding::Shift_JIS)]}
   end
 
+  def test_match_data_named_captures
+    assert_equal({'a' => '1', 'b' => '2', 'c' => nil}, /^(?<a>.)(?<b>.)(?<c>.)?/.match('12').named_captures)
+    assert_equal({'a' => '1', 'b' => '2', 'c' => '3'}, /^(?<a>.)(?<b>.)(?<c>.)?/.match('123').named_captures)
+    assert_equal({'a' => '1', 'b' => '2', 'c' => ''}, /^(?<a>.)(?<b>.)(?<c>.?)/.match('12').named_captures)
+
+    assert_equal({'a' => 'x'}, /(?<a>x)|(?<a>y)/.match('x').named_captures)
+    assert_equal({'a' => 'y'}, /(?<a>x)|(?<a>y)/.match('y').named_captures)
+
+    assert_equal({'a' => '1', 'b' => '2'}, /^(.)(?<a>.)(?<b>.)/.match('012').named_captures)
+    assert_equal({'a' => '2'}, /^(?<a>.)(?<a>.)/.match('12').named_captures)
+
+    assert_equal({}, /^(.)/.match('123').named_captures)
+  end
+
   def test_assign_named_capture
     assert_equal("a", eval('/(?<foo>.)/ =~ "a"; foo'))
+    assert_equal(nil, eval('/(?<@foo>.)/ =~ "a"; defined?(@foo)'))
     assert_equal("a", eval('foo = 1; /(?<foo>.)/ =~ "a"; foo'))
     assert_equal("a", eval('1.times {|foo| /(?<foo>.)/ =~ "a"; break foo }'))
     assert_nothing_raised { eval('/(?<Foo>.)/ =~ "a"') }
@@ -186,6 +211,15 @@ class TestRegexp < Test::Unit::TestCase
     assert_not_include(local_variables, :nil, "[ruby-dev:32675]")
   end
 
+  def test_assign_named_capture_trace
+    bug = '[ruby-core:79940] [Bug #13287]'
+    assert_normal_exit("#{<<-"begin;"}\n#{<<-"end;"}", bug)
+    begin;
+      / (?<foo>.*)/ =~ "bar" &&
+        true
+    end;
+  end
+
   def test_match_regexp
     r = /./
     m = r.match("a")
@@ -355,15 +389,46 @@ class TestRegexp < Test::Unit::TestCase
 
   def test_match_aref
     m = /(...)(...)(...)(...)?/.match("foobarbaz")
+    assert_equal("foobarbaz", m[0])
     assert_equal("foo", m[1])
+    assert_equal("foo", m[-4])
+    assert_nil(m[-1])
+    assert_nil(m[-11])
+    assert_nil(m[-11, 1])
+    assert_nil(m[-11..1])
+    assert_nil(m[5])
+    assert_nil(m[9])
     assert_equal(["foo", "bar", "baz"], m[1..3])
+    assert_equal(["foo", "bar", "baz"], m[1, 3])
+    assert_equal([], m[3..1])
+    assert_equal([], m[3, 0])
+    assert_equal(nil, m[3, -1])
+    assert_equal(nil, m[9, 1])
+    assert_equal(["baz"], m[3, 1])
+    assert_equal(["baz", nil], m[3, 5])
     assert_nil(m[5])
     assert_raise(IndexError) { m[:foo] }
+    assert_raise(TypeError) { m[nil] }
   end
 
   def test_match_values_at
+    idx = Object.new
+    def idx.to_int; 2; end
     m = /(...)(...)(...)(...)?/.match("foobarbaz")
     assert_equal(["foo", "bar", "baz"], m.values_at(1, 2, 3))
+    assert_equal(["foo", "bar", "baz"], m.values_at(1..3))
+    assert_equal(["foo", "bar", "baz", nil, nil], m.values_at(1..5))
+    assert_equal([], m.values_at(3..1))
+    assert_equal([nil, nil, nil, nil, nil], m.values_at(5..9))
+    assert_equal(["bar"], m.values_at(idx))
+    assert_raise(RangeError){ m.values_at(-11..1) }
+    assert_raise(TypeError){ m.values_at(nil) }
+
+    m = /(?<a>\d+) *(?<op>[+\-*\/]) *(?<b>\d+)/.match("1 + 2")
+    assert_equal(["1", "2", "+"], m.values_at(:a, 'b', :op))
+    assert_equal(["+"], m.values_at(idx))
+    assert_raise(TypeError){ m.values_at(nil) }
+    assert_raise(IndexError){ m.values_at(:foo) }
   end
 
   def test_match_string
@@ -479,6 +544,30 @@ class TestRegexp < Test::Unit::TestCase
     $_ = nil; assert_nil(~/./)
   end
 
+  def test_match_p
+    /backref/ =~ 'backref'
+    # must match here, but not in a separate method, e.g., assert_send,
+    # to check if $~ is affected or not.
+    assert_equal(false, //.match?(nil))
+    assert_equal(true, //.match?(""))
+    assert_equal(true, /.../.match?(:abc))
+    assert_raise(TypeError) { /.../.match?(Object.new) }
+    assert_equal(true, /b/.match?('abc'))
+    assert_equal(true, /b/.match?('abc', 1))
+    assert_equal(true, /../.match?('abc', 1))
+    assert_equal(true, /../.match?('abc', -2))
+    assert_equal(false, /../.match?("abc", -4))
+    assert_equal(false, /../.match?("abc", 4))
+    assert_equal(true, /../.match?("\u3042xx", 1))
+    assert_equal(false, /../.match?("\u3042x", 1))
+    assert_equal(true, /\z/.match?(""))
+    assert_equal(true, /\z/.match?("abc"))
+    assert_equal(true, /R.../.match?("Ruby"))
+    assert_equal(false, /R.../.match?("Ruby", 1))
+    assert_equal(false, /P.../.match?("Ruby"))
+    assert_equal('backref', $&)
+  end
+
   def test_eqq
     assert_equal(false, /../ === nil)
   end
@@ -578,9 +667,7 @@ class TestRegexp < Test::Unit::TestCase
     key = "\u{3042}"
     [Encoding::UTF_8, Encoding::Shift_JIS, Encoding::EUC_JP].each do |enc|
       idx = key.encode(enc)
-      EnvUtil.with_default_external(enc) do
-        test.call {|m| assert_raise_with_message(IndexError, /#{idx}/, bug10877) {m[idx]} }
-      end
+      test.call {|m| assert_raise_with_message(IndexError, /#{idx}/, bug10877) {m[idx]} }
     end
     test.call {|m| assert_equal(/a/, m.regexp) }
     test.call {|m| assert_equal("abc", m.string) }
@@ -590,6 +677,7 @@ class TestRegexp < Test::Unit::TestCase
     test.call {|m| assert_equal([0, 1], m.offset(0)) }
     test.call {|m| assert_equal([], m.captures) }
     test.call {|m| assert_equal([], m.names) }
+    test.call {|m| assert_equal({}, m.named_captures) }
     test.call {|m| assert_equal(/a/.match("abc"), m) }
     test.call {|m| assert_equal(/a/.match("abc").hash, m.hash) }
     test.call {|m| assert_equal("bc", m.post_match) }
@@ -848,6 +936,29 @@ class TestRegexp < Test::Unit::TestCase
     assert_no_match(/[[:ascii:]]/, "\x80\xFF")
   end
 
+  def test_cclass_R
+    assert_match /\A\R\z/, "\r"
+    assert_match /\A\R\z/, "\n"
+    assert_match /\A\R\z/, "\r\n"
+  end
+
+  def test_cclass_X
+    assert_match /\A\X\z/, "\u{20 200d}"
+    assert_match /\A\X\z/, "\u{600 600}"
+    assert_match /\A\X\z/, "\u{600 20}"
+    assert_match /\A\X\z/, "\u{261d 1F3FB}"
+    assert_match /\A\X\z/, "\u{1f600}"
+    assert_match /\A\X\z/, "\u{20 308}"
+    assert_match /\A\X\X\z/, "\u{a 308}"
+    assert_match /\A\X\X\z/, "\u{d 308}"
+    assert_match /\A\X\z/, "\u{1F477 1F3FF 200D 2640 FE0F}"
+    assert_match /\A\X\z/, "\u{1F468 200D 1F393}"
+    assert_match /\A\X\z/, "\u{1F46F 200D 2642 FE0F}"
+    assert_match /\A\X\z/, "\u{1f469 200d 2764 fe0f 200d 1f469}"
+
+    assert_warning('') {/\X/ =~ "\u{a0}"}
+  end
+
   def test_backward
     assert_equal(3, "foobar".rindex(/b.r/i))
     assert_equal(nil, "foovar".rindex(/b.r/i))
@@ -873,6 +984,7 @@ class TestRegexp < Test::Unit::TestCase
     assert_raise(TypeError) { Regexp.allocate.names }
     assert_raise(TypeError) { Regexp.allocate.named_captures }
 
+    assert_raise(TypeError) { MatchData.allocate.hash }
     assert_raise(TypeError) { MatchData.allocate.regexp }
     assert_raise(TypeError) { MatchData.allocate.names }
     assert_raise(TypeError) { MatchData.allocate.size }
@@ -961,9 +1073,13 @@ class TestRegexp < Test::Unit::TestCase
     h = {a => 42}
     assert_equal(42, h[b], '[ruby-core:24748]')
     assert_match(/#<TestRegexp::MatchData_\u{3042}:/, MatchData_A.allocate.inspect)
+
+    h = /^(?<@time>\d+): (?<body>.*)/.match("123456: hoge fuga")
+    assert_equal("123456", h["@time"])
+    assert_equal("hoge fuga", h["body"])
   end
 
-  def test_regexp_poped
+  def test_regexp_popped
     assert_nothing_raised { eval("a = 1; /\#{ a }/; a") }
     assert_nothing_raised { eval("a = 1; /\#{ a }/o; a") }
   end
@@ -1014,7 +1130,7 @@ class TestRegexp < Test::Unit::TestCase
   def test_error_message_on_failed_conversion
     bug7539 = '[ruby-core:50733]'
     assert_equal false, /x/=== 42
-    assert_raise_with_message(TypeError, 'no implicit conversion of Fixnum into String', bug7539) {
+    assert_raise_with_message(TypeError, 'no implicit conversion of Integer into String', bug7539) {
       Regexp.quote(42)
     }
   end
@@ -1063,7 +1179,7 @@ class TestRegexp < Test::Unit::TestCase
   end
 
   def test_once_multithread
-    m = Mutex.new
+    m = Thread::Mutex.new
     pr3 = proc{|i|
       /#{m.unlock; sleep 0.5; i}/o
     }
@@ -1099,6 +1215,36 @@ class TestRegexp < Test::Unit::TestCase
     RUBY
   end
 
+  def test_invalid_free_at_parse_depth_limit_over
+    assert_separately([], "#{<<-"begin;"}\n#{<<-"end;"}")
+    begin;
+      begin
+        require '-test-/regexp'
+      rescue LoadError
+      else
+        bug = '[ruby-core:79624] [Bug #13234]'
+        Bug::Regexp.parse_depth_limit = 10
+        src = "[" * 100
+        3.times do
+          assert_raise_with_message(RegexpError, /parse depth limit over/, bug) do
+            Regexp.new(src)
+          end
+        end
+      end
+    end;
+  end
+
+  def test_absent
+    assert_equal(0, /(?~(a|c)c)/ =~ "abb")
+    assert_equal("abb", $&)
+
+    assert_equal(0, /\/\*((?~\*\/))\*\// =~ "/*abc*def/xyz*/ /* */")
+    assert_equal("abc*def/xyz", $1)
+
+    assert_equal(0, /(?~(a)c)/ =~ "abb")
+    assert_nil($1)
+  end
+
   # This assertion is for porting x2() tests in testpy.py of Onigmo.
   def assert_match_at(re, str, positions, msg = nil)
     re = Regexp.new(re) unless re.is_a?(Regexp)
